@@ -16,8 +16,8 @@ import threading
 # import _thread  使用_thread会报错，坑！
 
 
-__versinon__ = '1.2.7'
-__last_modified__ = '2023/7/2'
+__versinon__ = '1.3.3'
+__last_modified__ = '2023/8/20'
 
 GPIO.setwarnings(False)
 GPIO.setmode(GPIO.BCM)
@@ -179,9 +179,28 @@ class XGOEDU():
     '''
     x1,y1,x2,y2为定义给定边框的两个点,angle0为初始角度,angle1为终止角度
     '''
-    def lcd_circle(self,x1,y1,x2,y2,angle0,angle1,color="WHITE",width=2):
+    def lcd_arc(self,x1,y1,x2,y2,angle0,angle1,color="WHITE",width=2):
         self.draw.arc((x1,y1,x2,y2),angle0,angle1,fill=color ,width=width)
         self.display.ShowImage(self.splash)
+
+    #绘画圆:  根据圆形点和半径画圆
+    '''
+    center_x, center_y 圆心点坐标
+    radius 圆半径长度 mm
+    
+    '''
+    def lcd_circle(self,center_x, center_y, radius, color, width=2):
+        # Calculate the bounding box for the circle
+        x1 = center_x - radius
+        y1 = center_y - radius
+        x2 = center_x + radius
+        y2 = center_y + radius
+    
+        # Call lcd_circle() function to draw the circle
+        self.lcd_arc(x1, y1, x2, y2, 0, 360, color=color, width=width)
+  
+
+    
     #绘画矩形
     '''
     x1,y1为初始点坐标,x2,y2为对角线终止点坐标
@@ -192,6 +211,7 @@ class XGOEDU():
     #清除屏幕
     def lcd_clear(self):
         self.splash = Image.new("RGB",(320,240),"black")
+        self.draw = ImageDraw.Draw(self.splash)
         self.display.ShowImage(self.splash)
     #显示图片
     '''
@@ -211,6 +231,91 @@ class XGOEDU():
             self.font = ImageFont.truetype("/home/pi/model/msyh.ttc",fontsize)
         self.draw.text((x,y),content,fill=color,font=self.font)
         self.display.ShowImage(self.splash)
+    #流式显示所有文字
+    '''
+    x1,y1为初始点坐标,content为内容
+    遇到回车符自动换行，遇到边缘换行，一页满了自动清屏，2,2开始继续显示
+    '''
+    def display_text_on_screen(content, color, start_x=2, start_y=2, font_size=20, screen_width=320, screen_height=240):
+        XGO_edu = XGOEDU()  # 实例化edu
+    
+        # 计算每行可显示字符的数量和行数
+        char_width = font_size +1  #// 2
+        chars_per_line = screen_width // char_width
+        lines = screen_height // char_width
+    
+        # 拆分内容为逐个字符的列表
+        chars = list(content)
+     
+        # 处理换行符
+        line_break_indices = [i for i, char in enumerate(chars) if char == '\n']
+    
+    
+        # 计算总行数和页数
+        total_lines = len(chars) // chars_per_line + 1
+        total_pages = (total_lines - 1+len(line_break_indices)) // lines + 1
+    
+        # 清屏
+        self.display.clear()
+    
+        # 逐行显示文字
+        current_page = 1
+        current_line = 1
+        current_char = 0
+    
+        while current_page <= total_pages or  current_char < len(chars) :
+            self.display.clear()
+            # 计算当前页要显示的行数
+            if current_page < total_pages or  current_char < len(chars) :
+                lines_to_display = lines
+            else:
+                lines_to_display = (total_lines - 1) % lines + 1
+    
+            current_line = 1
+            # 显示当前页的内容
+            for line in range(lines_to_display):
+                current_x = start_x
+                current_y = start_y + current_line * char_width # font_size
+                current_line +=1
+                if current_line >= lines:
+                    break
+    
+                # 显示当前行的文字
+                for _ in range(chars_per_line):
+                    # 检查是否所有字符都已显示完毕
+                    if current_char >= len(chars):
+                        break
+    
+                    char = chars[current_char]
+                    if char == '\n':
+                        current_x = start_x
+                        current_y = start_y + current_line * char_width # font_size
+                        current_line +=1
+                       
+                        XGO_edu.lcd_text(current_x, current_y, char, color, font_size)
+                        current_char += 1
+                        break  # continue
+    
+                    lcd_text(current_x, current_y, char, color, font_size)
+                    current_x += char_width
+                    current_char += 1
+    
+                # 检查是否所有字符都已显示完毕
+                if current_char >= len(chars):
+                    break
+    
+            # 更新当前页和当前行
+            current_page += 1
+            current_line += lines_to_display
+    
+            # 等待显示时间或手动触发翻页
+            # 这里可以根据需要添加适当的延时代码或触发翻页的机制
+    
+        # 如果内容超过一屏幕，则清屏
+        # if total_lines > lines:
+        if current_page < total_pages:
+            self.display.clear()
+    
     #key_value
     '''
     a左上按键
@@ -426,6 +531,94 @@ class XGOEDU():
                 XGOEDU.lcd_clear(self)
                 time.sleep(0.5)
                 break
+    '''
+    骨骼识别
+    '''
+    def posenetRecognition(self,target="camera"):
+        import mediapipe as mp
+        mp_pose = mp.solutions.pose
+        ges = ''
+        mp_drawing = mp.solutions.drawing_utils
+        mp_drawing_styles = mp.solutions.drawing_styles
+        mp_holistic = mp.solutions.holistic
+        joint_list = [[24,26,28], [23,25,27], [14,12,24], [13,11,23]]  # leg&arm
+        if target=="camera":
+            self.open_camera()
+            success,image = self.cap.read()
+        else:
+            image=np.array(Image.open(target))
+
+        with mp_pose.Pose(
+                min_detection_confidence=0.5,
+                min_tracking_confidence=0.5) as pose:
+
+                image.flags.writeable = False
+                image = cv2.cvtColor(image, cv2.COLOR_BGR2RGB)
+                results = pose.process(image)
+
+                # Draw the pose annotation on the image.
+                image.flags.writeable = True
+                image = cv2.cvtColor(image, cv2.COLOR_RGB2BGR)
+                mp_drawing.draw_landmarks(
+                    image,
+                    results.pose_landmarks,
+                    mp_pose.POSE_CONNECTIONS,
+                    landmark_drawing_spec=mp_drawing_styles.get_default_pose_landmarks_style())
+                # Flip the image horizontally for a selfie-view display.
+                
+                if results.pose_landmarks:
+                    RHL = results.pose_landmarks
+                    angellist=[]
+                    for joint in joint_list:
+                        a = np.array([RHL.landmark[joint[0]].x, RHL.landmark[joint[0]].y])
+                        b = np.array([RHL.landmark[joint[1]].x, RHL.landmark[joint[1]].y])
+                        c = np.array([RHL.landmark[joint[2]].x, RHL.landmark[joint[2]].y])
+                        radians_fingers = np.arctan2(c[1] - b[1], c[0] - b[0]) - np.arctan2(a[1] - b[1], a[0] - b[0])
+                        angle = np.abs(radians_fingers * 180.0 / np.pi) 
+                        if angle > 180.0:
+                            angle = 360 - angle
+                        #cv2.putText(image, str(round(angle, 2)), tuple(np.multiply(b, [640, 480]).astype(int)),cv2.FONT_HERSHEY_SIMPLEX, 0.5, (255, 255, 255), 2, cv2.LINE_AA)
+                        angellist.append(angle)
+                else:
+                    angellist=[]
+                print(angellist)
+                b,g,r = cv2.split(image)
+                image = cv2.merge((r,g,b))
+                image = cv2.flip(image, 1)
+                try:
+                    ges=str(int(angellist[0]))+'|'+str(int(angellist[1]))+'|'+str(int(angellist[2]))+'|'+str(int(angellist[3]))
+                except:
+                    ges=' '
+                cv2.putText(image,ges,(10,220),cv2.FONT_HERSHEY_SIMPLEX, 0.5, (255, 255, 255), 2, cv2.LINE_AA)
+                imgok = Image.fromarray(image)
+                self.display.ShowImage(imgok)
+
+
+        # datas = self.hand.run(image)
+        # b,g,r = cv2.split(image)
+        # image = cv2.merge((r,g,b))
+        # #image = cv2.flip(image,1)
+        # for data in datas:
+        #     rect = data['rect']
+        #     right_left = data['right_left']
+        #     center = data['center']
+        #     dlandmark = data['dlandmark']
+        #     hand_angle = data['hand_angle']
+        #     XGOEDU.rectangle(self,image,rect,"#33cc00",2)
+        #     #XGOEDU.text(self,image,right_left,center,2,"#cc0000",5)
+        #     if right_left == 'L':
+        #         XGOEDU.text(self,image,hand_pos(hand_angle),(180,80),1.5,"#33cc00",2)
+        #     elif right_left == 'R':
+        #         XGOEDU.text(self,image,hand_pos(hand_angle),(50,80),1.5,"#ff0000",2)
+        #     ges = hand_pos(hand_angle)
+        #     for i in dlandmark:
+        #         XGOEDU.circle(self,image,i,3,"#ff9900",-1)
+        # imgok = Image.fromarray(image)
+        # self.display.ShowImage(imgok)
+        if angellist==[]:
+            return None
+        else:
+            return angellist
 
     '''
     手势识别
@@ -756,36 +949,50 @@ class XGOEDU():
             of.write(result_str)
 
         if has_error:
-            if (IS_PY3):
-                result_str = str(result_str, 'utf-8')
+            result_str = str(result_str, 'utf-8')
             print("tts api  error:" + result_str)
 
         print("result saved as :" + save_file)
 
         self.xgoSpeaker("result.wav")
+
+    def cv2AddChineseText(self,img, text, position, textColor=(0, 255, 0), textSize=30):
+        if (isinstance(img, np.ndarray)):  
+            img = Image.fromarray(cv2.cvtColor(img, cv2.COLOR_BGR2RGB))
+        draw = ImageDraw.Draw(img)
+        fontStyle = ImageFont.truetype(
+            "/home/pi/model/msyh.ttc", textSize, encoding="utf-8")
+        draw.text(position, text, textColor, font=fontStyle)
+        return cv2.cvtColor(np.asarray(img), cv2.COLOR_RGB2BGR)
     
     def QRRecognition(self,target="camera"):
         import pyzbar.pyzbar as pyzbar
         if target=="camera":
-            self.xgoTakePhoto(filename="photo")
-            path="/home/pi/xgoPictures/photo.jpg"
-            img=cv2.imread(path)
+            self.open_camera()
+            success,img = self.cap.read()
         else:
-            img=cv2.imread("/home/pi/xgoPictures/"+target)
+            path="/home/pi/xgoPictures/"
+            img=np.array(Image.open(path+target))
+     
         barcodes = pyzbar.decode(img) 
         result=[]
         for barcode in barcodes:
             barcodeData = barcode.data.decode("utf-8")
             barcodeType = barcode.type
             result.append(barcodeData)
+            text = "{} ({})".format(barcodeData, barcodeType)
+            img=self.cv2AddChineseText(img,text, (10, 30),(0, 255, 0), 30)
         try:
             re=result[0]
         except:
             result=[]
-        self.xgoCamera(False)
+        b,g,r = cv2.split(img)
+        img = cv2.merge((r,g,b))
+        imgok = Image.fromarray(img)
+        self.display.ShowImage(imgok)
         return result
 
-    def ColorRecognitio(self,target="camera",mode='R'):
+    def ColorRecognition(self,target="camera",mode='R'):
         color_x = 0
         color_y = 0
         color_radius = 0
@@ -803,13 +1010,13 @@ class XGOEDU():
             color_lower = np.array([26, 43, 46])
             color_upper = np.array([34, 255, 255])
         if target=="camera":
-            self.xgoTakePhoto(filename="photo")
-            path="/home/pi/xgoPictures/photo.jpg"
-            img=cv2.imread(path)
+            self.open_camera()
+            success,frame = self.cap.read()
         else:
-            img=cv2.imread("/home/pi/xgoPictures/"+target)
-        frame_ = cv2.GaussianBlur(img,(5,5),0)                    
-        hsv = cv2.cvtColor(img,cv2.COLOR_BGR2HSV)
+            path="/home/pi/xgoPictures/"
+            frame=np.array(Image.open(path+target))
+        frame_ = cv2.GaussianBlur(frame,(5,5),0)                    
+        hsv = cv2.cvtColor(frame,cv2.COLOR_BGR2HSV)
         mask = cv2.inRange(hsv,color_lower,color_upper)  
         mask = cv2.erode(mask,None,iterations=2)
         mask = cv2.dilate(mask,None,iterations=2)
@@ -819,30 +1026,84 @@ class XGOEDU():
         if len(cnts) > 0:
             cnt = max (cnts, key = cv2.contourArea)
             (color_x,color_y),color_radius = cv2.minEnclosingCircle(cnt)
+            cv2.circle(frame,(int(color_x),int(color_y)),int(color_radius),(255,0,255),2)  
+        cv2.putText(frame, "X:%d, Y%d" % (int(color_x), int(color_y)), (40,40), cv2.FONT_HERSHEY_SIMPLEX, 0.8, (255,255,0), 3)
 
-        self.xgoCamera(False)
+        b,g,r = cv2.split(frame)
+        img = cv2.merge((r,g,b))
+        imgok = Image.fromarray(img)
+        self.display.ShowImage(imgok)
 
         return ((color_x,color_y),color_radius)
 
-    def CircleRecognition(self,target="camera"):
-        if target=="camera":
-            self.xgoTakePhoto(filename="photo")
-            path="/home/pi/xgoPictures/photo.jpg"
-            img = cv2.imread(path, cv2.IMREAD_GRAYSCALE)     # 0 转为灰度图像
-            #img2 = cv2.imread(path, cv2.IMREAD_COLOR)     # 1 为彩色图像
+    def cap_color_mask(self,position=None, scale=25, h_error=20, s_limit=[90, 255], v_limit=[90, 230]):
+        if position is None:
+            position = [160, 100]
+        count = 0
+        self.open_camera()
+        while True:
+            if self.xgoButton("c"):   
+                break
+            success,frame = self.cap.read()
+            b,g,r = cv2.split(frame)
+            frame_bgr = cv2.merge((r,g,b))
+            hsv = cv2.cvtColor(frame_bgr, cv2.COLOR_BGR2HSV)
+            h, s, v = cv2.split(hsv)
+            color = np.mean(h[position[1]:position[1] + scale, position[0]:position[0] + scale])
+            if self.xgoButton("b") and count == 0:
+                count += 1
+                color = np.mean(h[position[1]:position[1] + scale, position[0]:position[0] + scale])
+                color_lower = [max(color - h_error, 0), s_limit[0], v_limit[0]]
+                color_upper = [min(color + h_error, 255), s_limit[1], v_limit[1]]
+                return [color_lower, color_upper]
+
+            if count == 0:
+                cv2.rectangle(frame, (position[0], position[1]), (position[0] + scale, position[1] + scale),
+                            (255, 255, 255), 2)
+                cv2.putText(frame, 'press button B', (40, 40), cv2.FONT_HERSHEY_SIMPLEX, 0.7, (0, 0, 255), 2)
+            b,g,r = cv2.split(frame)
+            img = cv2.merge((r,g,b))
+            imgok = Image.fromarray(img)
+            self.display.ShowImage(imgok)
+    
+    def filter_img(self,frame,color):
+        b,g,r = cv2.split(frame)
+        frame_bgr = cv2.merge((r,g,b))
+        hsv = cv2.cvtColor(frame_bgr, cv2.COLOR_BGR2HSV)
+        if isinstance(color, list):
+            color_lower = np.array(color[0])
+            color_upper = np.array(color[1])
         else:
-            img=cv2.imread("/home/pi/xgoPictures/"+target,cv2.IMREAD_GRAYSCALE)
-        numpy_img = cv2.adaptiveThreshold(img, 255, cv2.ADAPTIVE_THRESH_GAUSSIAN_C, cv2.THRESH_BINARY, 11, 15)   # 自动阈值二值化
-        #                                                      圆心距 canny阈值    投票数      最小半径       最大半径
-        circles = cv2.HoughCircles(img, cv2.HOUGH_GRADIENT, 1, 18,   param1=100, param2=80,  minRadius=40, maxRadius=150)
-        arr1 = np.zeros([0, 2], dtype=int)                      # 创建一个0行, 2列的空数组
-        re=[]
-        if circles is not None:
-            circles = np.uint16(np.around(circles))   
-            for i in circles[0, :]:
-                re.append(((i[0], i[1]), i[2]))
-        self.xgoCamera(False)
-        return re
+            color_upper, color_lower = get_color_mask(color)
+        mask = cv2.inRange(hsv, color_lower, color_upper)
+        img_mask = cv2.bitwise_and(frame, frame, mask=mask)
+        return img_mask
+
+    def BallRecognition(self,color_mask,target="camera",p1=36, p2=15, minR=6, maxR=35):
+        x=y=ra=0
+        if target=="camera":
+            self.open_camera()
+            success,image = self.cap.read()
+        else:
+            path="/home/pi/xgoPictures/"
+            image=np.array(Image.open(path+target))
+
+        frame_mask=self.filter_img(image, color_mask)
+        
+        img = cv2.medianBlur(frame_mask, 5)
+        img = cv2.cvtColor(img,cv2.COLOR_BGR2GRAY)
+        
+        circles = cv2.HoughCircles(img, cv2.HOUGH_GRADIENT, 1, 20, param1=p1, param2=p2, minRadius=minR,maxRadius=maxR)
+        b,g,r = cv2.split(image)
+        image = cv2.merge((r,g,b))
+        if circles is not None and len(circles[0]) == 1:
+            param = circles[0][0]
+            x, y, ra = int(param[0]), int(param[1]), int(param[2])
+            cv2.circle(image, (x, y), ra, (255, 255, 255), 2)
+            cv2.circle(image, (x, y), 2, (255, 255, 255), 2)
+        imgok = Image.fromarray(image)
+        self.display.ShowImage(imgok)
+        return x,y,ra
 
 
 
@@ -1175,4 +1436,4 @@ class face_detection():
             'right_ear':(int(keypoint4.x),int(keypoint4.y)),
             'left_ear':(int(keypoint5.x),int(keypoint5.y)),
             }
-        return data
+        return data
\ No newline at end of file
